package com.tianyi.sync.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.tianyi.sync.mapper.master.CashFlowMapper;
import com.tianyi.sync.mapper.slave.FallSlaveMapper;
import com.tianyi.sync.mapper.slave2.FixedAsstesResidualValueMapper;
import com.tianyi.sync.mapper.slave3.FixedAssetsMapper;
import com.tianyi.sync.model.FixedAssetsModel;
import com.tianyi.sync.model.FixedAssetsResidualValeModel;
import com.tianyi.sync.service.FixedAssetsService;
import com.tianyi.sync.utils.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StopWatch;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.Period;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

@Service
public class FixedAssetsServiceImpl implements FixedAssetsService {

    private final static Logger logger = LoggerFactory.getLogger(FixedAssetsServiceImpl.class);

    private final FixedAssetsMapper fixedAssetsMapper;

    private final FixedAsstesResidualValueMapper fixedResidualValueMapper;

    public FixedAssetsServiceImpl(FixedAssetsMapper fixedAssetsMapper, CashFlowMapper cashFlowMapper, FixedAsstesResidualValueMapper fixedResidualValueMapper, FallSlaveMapper fallSlaveMapper) {
        this.fixedAssetsMapper = fixedAssetsMapper;
        this.fixedResidualValueMapper = fixedResidualValueMapper;
    }

    @Override
    public Object residualVaule() {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        String period = DateUtils.getYearAndMonthLast();
        //String period="2024-02";
        String lastDate = fixedResidualValueMapper.getLastDate();
        List<FixedAssetsModel> fixedAssetsModelList = fixedAssetsMapper.getFixedAssetsInfo(lastDate);
        List<FixedAssetsResidualValeModel> fixedAssetsResidualValeModelList = new ArrayList<>();
        for (FixedAssetsModel model : fixedAssetsModelList) {
            FixedAssetsResidualValeModel fixedAssetsResidualValeModel = new FixedAssetsResidualValeModel();
            //创建人
            fixedAssetsResidualValeModel.setOperMan(model.getCreator());
            //创建时间
            fixedAssetsResidualValeModel.setOperTime(model.getCreateTime());
            //会计主体ID
            fixedAssetsResidualValeModel.setAccentityId(model.getAccentityId());
            //固定资产编码
            fixedAssetsResidualValeModel.setPropertyCode(model.getCode());
            //固定资产名称
            fixedAssetsResidualValeModel.setName(model.getName());
            //增加方式
            fixedAssetsResidualValeModel.setAddType(model.getAddWayIdName());
            //折旧方法名称
            fixedAssetsResidualValeModel.setLoseTypeName(model.getDeprMethodIdName());
            //本币原值
            BigDecimal oriValue = model.getBodiesOriValue();
            fixedAssetsResidualValeModel.setPrice(oriValue);
            //残值率
            BigDecimal rate = new BigDecimal("0.05");
            fixedAssetsResidualValeModel.setLoseRate(rate);
            //净残值
            BigDecimal leftPrice = oriValue.multiply(rate);
            fixedAssetsResidualValeModel.setLeftPrice(leftPrice);
            //总使用年限（年月转换为数，5年3个月=63个月）
            fixedAssetsResidualValeModel.setUserPeriod((Integer.parseInt(model.getLifeYear()) * 12) + Integer.parseInt(model.getLifeMonth()));
            //开始使用日期
            String beginTime = model.getBeginTime();
            fixedAssetsResidualValeModel.setStartUseTime(beginTime);
            //截至上年末已使用期数(月)
            LocalDate startDate = LocalDate.parse(beginTime);
            LocalDate endDate = LocalDate.now().minusYears(1).withMonth(12).withDayOfMonth(31);
            Period periodBetween = Period.between(startDate, endDate);
            int preYearUseTimeInMonths = (periodBetween.getYears() * 12) + periodBetween.getMonths();
            fixedAssetsResidualValeModel.setPreYearUseTime(preYearUseTimeInMonths);
            //月折旧额
            BigDecimal monthLosePrice = (oriValue.subtract(leftPrice)).divide(new BigDecimal(fixedAssetsResidualValeModel.getUserPeriod()), 2, RoundingMode.HALF_UP);
            fixedAssetsResidualValeModel.setMonthLosePrice(monthLosePrice);
            //截止上年末累计计提折旧额
            //截至上年末已使用期数(月)>=总使用期限？总使用期限*月折旧额：截止上年末已使用期数(月)*月折旧额
            if (fixedAssetsResidualValeModel.getPreYearUseTime() >= fixedAssetsResidualValeModel.getUserPeriod()) {
                //本应该为总使用期限*月折旧额但是因为会有小数计算，会存在总使用期限*月折旧额与本币原值-净残值不相等的情况，所以取本币原值-净残值，会计上叫做平
                fixedAssetsResidualValeModel.setTotalLosePrice(fixedAssetsResidualValeModel.getPrice().subtract(fixedAssetsResidualValeModel.getLeftPrice()));
            } else {
                BigDecimal totalLosePrice = new BigDecimal(preYearUseTimeInMonths).multiply(monthLosePrice);
                fixedAssetsResidualValeModel.setTotalLosePrice(totalLosePrice);
            }

            //本年折旧期数
            /**
             * 1.如果处置状态是已处置，判断处置日期是否为操作的当年
             *  是当年
             *      1)同一年，取相差月份
             *      2)不同年，取当前年月份
             *  不是当年
             *      1)取0
             * 2.处置状态为非处置，取总使用期限-截止上年年末已使用期数=a
             *  1)a>=12个月取12
             *  2)0>a<12取月份
             *  3)a<0取0
             */
            int monthCount = 0;
            if (("已处置").equals(model.getUsageStateIdName())) {
                LocalDate begin = LocalDate.parse(beginTime);
                LocalDate end = LocalDate.parse(model.getAuditTime(), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                //如果开始使用日期月的号数大于处置日期月的号数，会少算一个月，所以设置开始使用日期月的号数小于处置日期月的号数
                if (begin.getDayOfMonth() > end.getDayOfMonth()) {
                    begin.withDayOfMonth(1);
                    end.withDayOfMonth(end.getMonth().maxLength());
                }
                int currentYear = LocalDate.now().getYear();
                if (currentYear == end.getYear()) {
                    if (begin.getYear() == end.getYear()) {
                        monthCount = Period.between(begin, end).getMonths();
                    } else {
                        monthCount = end.getMonthValue();
                    }
                }

            } else {
                monthCount = fixedAssetsResidualValeModel.getUserPeriod() - fixedAssetsResidualValeModel.getPreYearUseTime();
                if (monthCount > 12) {
                    monthCount = 12;
                }

                if (monthCount < 0) {
                    monthCount = 0;
                }
            }
            fixedAssetsResidualValeModel.setYearLoseCount(BigDecimal.valueOf(monthCount));

            //本币本年折旧额
            fixedAssetsResidualValeModel.setYearLosePrice(fixedAssetsResidualValeModel.getYearLoseCount().multiply(monthLosePrice));

            //截止本年末累计计提折旧额
            fixedAssetsResidualValeModel.setTotalFinishLosePrice(fixedAssetsResidualValeModel.getTotalLosePrice().add(fixedAssetsResidualValeModel.getYearLosePrice()));

            //应计折旧额
            fixedAssetsResidualValeModel.setTreatLosePrice(fixedAssetsResidualValeModel.getPrice().subtract(fixedAssetsResidualValeModel.getLeftPrice()).subtract(fixedAssetsResidualValeModel.getTotalLosePrice()));

            //处置状态
            fixedAssetsResidualValeModel.setTreatState(model.getUsageStateIdName());
            if (("已处置").equals(model.getUsageStateIdName())) {
                fixedAssetsResidualValeModel.setTreatTime(model.getAuditTime());
            }
            //资产用途
            fixedAssetsResidualValeModel.setPropertyPurpose(model.getPropertyPurpose());

//            UUID uuid = UUID.randomUUID();
//            fixedAssetsResidualValeModel.setId(uuid.toString());

            fixedAssetsResidualValeModel.setYtenantId("dmrn1j0i");
            fixedAssetsResidualValeModelList.add(fixedAssetsResidualValeModel);
        }

        int save = 0;
        if (!CollectionUtils.isEmpty(fixedAssetsResidualValeModelList)) {
//            fixedResidualValueMapper.deleteResidualValue();
            save = fixedResidualValueMapper.save(fixedAssetsResidualValeModelList);
        }

        stopWatch.stop();
        logger.info("执行耗时:{}，保存数据{}条", stopWatch.getTotalTimeMillis(), save);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("msg", "保存成功!");
        return jsonObject;
    }
}
